﻿DROP user MSG_AQ_USER cascade;
CREATE USER MSG_AQ_USER IDENTIFIED BY aquser;
GRANT connect,resource to MSG_AQ_USER;

DROP user MSG_AQ_ADMIN cascade;
CREATE USER MSG_AQ_ADMIN IDENTIFIED BY aqadm;
GRANT connect,resource,AQ_ADMINISTRATOR_ROLE to MSG_AQ_ADMIN;
grant execute on dbms_aq to MSG_AQ_ADMIN;

create or replace
TYPE MSG_AQ_ADMIN.aq_message_type
AS OBJECT(
  business_id NUMBER,
  title       VARCHAR2(30),
  message     VARCHAR2(2000)
);
/
BEGIN 
  DBMS_AQADM.CREATE_QUEUE_TABLE(
    Queue_table        => 'MSG_AQ_ADMIN.MESG_IN_QUEUE_TBL',
    Queue_payload_type => 'MSG_AQ_ADMIN.AQ_MESSAGE_TYPE',
    Sort_list          => 'PRIORITY,ENQ_TIME'
  );
  DBMS_AQADM.CREATE_QUEUE(
    Queue_name          => 'MSG_AQ_ADMIN.MESG_IN_QUEUE',
    Queue_table         => 'MSG_AQ_ADMIN.MESG_IN_QUEUE_TBL',
    Queue_type          =>  0,
    Max_retries         =>  5,
    Retry_delay         =>  0,
    Retention_time      =>  3600,
    dependency_tracking =>  FALSE
  );
  DBMS_AQADM.START_QUEUE('MSG_AQ_ADMIN.MESG_IN_QUEUE');
END;
/
create or replace
PACKAGE MSG_AQ_ADMIN.AQ_UTIL_PKG AS 

  g_outbound_consumer         CONSTANT VARCHAR2(30) := 'MY_CONSUMER';
  g_inbound_consumer          CONSTANT VARCHAR2(30) := 'MY_MSG_CONSUMER';
  g_inbound_recipient_address CONSTANT VARCHAR2(64) := 'SCHEMA.QUEUE@DB';

  g_inbound_queue             CONSTANT VARCHAR2(64) := 'MSG_AQ_ADMIN.MESG_IN_QUEUE';
  g_outbound_queue            CONSTANT VARCHAR2(64) := 'MSG_AQ_ADMIN.MESG_IN_QUEUE';
  --g_outbound_queue            CONSTANT VARCHAR2(64) := 'MSG_AQ_ADMIN.MESG_OUT_QUEUE';
  g_outbound_ack_queue        CONSTANT VARCHAR2(64) := 'MSG_AQ_ADMIN.ACK_OUT_QUEUE';
  g_inbound_ack_queue         CONSTANT VARCHAR2(64) := 'MSG_AQ_ADMIN.ACK_IN_QUEUE';

  NO_MESSAGES      EXCEPTION;
  DEQUEUE_DISABLED EXCEPTION;
  ENQUEUE_DISABLED EXCEPTION;
  PRAGMA EXCEPTION_INIT(NO_MESSAGES, -25228);
  PRAGMA EXCEPTION_INIT (DEQUEUE_DISABLED, -25226);
  PRAGMA EXCEPTION_INIT (ENQUEUE_DISABLED, -25207);
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
  PROCEDURE enqueue(
    p_business_id IN NUMBER,
    p_title       IN VARCHAR2,
    p_message     IN VARCHAR2
  );
--------------------------------------------------------------------------------
  PROCEDURE dequeue(
    p_wait        IN NUMBER   DEFAULT NULL,
    p_filter      IN VARCHAR2 DEFAULT NULL,
    p_business_id OUT NUMBER,
    p_title       OUT VARCHAR2,
    p_message     OUT VARCHAR2
  );
--------------------------------------------------------------------------------
END;
/
create or replace
PACKAGE BODY MSG_AQ_ADMIN.AQ_UTIL_PKG AS
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
  PROCEDURE enqueue(
    p_business_id IN NUMBER,
    p_title       IN VARCHAR2,
    p_message     IN VARCHAR2
  ) AS
    l_enqueue_options     dbms_aq.enqueue_options_t;  
    l_message_properties  dbms_aq.message_properties_t;
    l_recipients          dbms_aq.aq$_recipient_list_t;
    l_message_handle      RAW(16);
    l_message             aq_message_type;  
  BEGIN
  
    l_message := aq_message_type(
      p_business_id,
      p_title,
      p_message
    );
    
    dbms_aq.enqueue(
      queue_name          => g_inbound_queue,
      enqueue_options     => l_enqueue_options,
      message_properties  => l_message_properties,
      payload             => l_message,
      msgid               => l_message_handle
    );
    COMMIT;
    -- Return message id
    -- p_mesg_id := l_message_handle;
    
  END;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
  PROCEDURE dequeue(
    p_wait        IN NUMBER   DEFAULT NULL,
    p_filter      IN VARCHAR2 DEFAULT NULL,
    p_business_id OUT NUMBER,
    p_title       OUT VARCHAR2,
    p_message     OUT VARCHAR2
  ) AS
    l_dequeue_options     dbms_aq.dequeue_options_t;
    l_message_properties  dbms_aq.message_properties_t;
    l_message_handle      RAW(16);
    l_message             aq_message_type;
  BEGIN
  
    -- If the wait parameter is null, we assume that no wait is required
    IF p_wait IS NULL THEN
      l_dequeue_options.wait := dbms_aq.no_wait;
    ELSE
      l_dequeue_options.wait := p_wait;
    END IF;
    
    -- If you use multi consumer queue, you must set the consumer name here
    -- dequeue_options.consumer_name := g_outbound_consumer;
    l_dequeue_options.navigation := dbms_aq.first_message;
    l_dequeue_options.deq_condition := p_filter;
    
    dbms_aq.dequeue(
      queue_name          => g_outbound_queue,
      dequeue_options     => l_dequeue_options,
      message_properties  => l_message_properties,
      payload             => l_message,
      msgid               => l_message_handle
    );
    
    p_business_id := l_message.business_id;
    p_title       := l_message.title;
    p_message     := l_message.message;

    COMMIT;
    
  EXCEPTION
  WHEN NO_MESSAGES THEN
    p_business_id := NULL;
    p_title       := NULL;
    p_message     := NULL;
  WHEN OTHERS THEN
  -- If you want some internal error handling, place it here before RAISE statement
  RAISE;
  END;
END;
/
create or replace
PROCEDURE MSG_AQ_ADMIN.enqueue_mesg(
    p_business_id IN NUMBER,
    p_title       IN VARCHAR2,
    p_message     IN VARCHAR2
  ) AS
BEGIN
  aq_util_pkg.enqueue(
    p_business_id => p_business_id,
    p_title       => p_title,
    p_message     => p_message
  );
END;
/
CREATE OR REPLACE VIEW MSG_AQ_ADMIN."MESG_IN_VW"
AS
  SELECT msg_state,
    enq_time,
    deq_time,
    msg_priority,
    q.user_data.business_id AS BUSINESS_ID,
    q.user_data.title       AS TITLE,
    q.user_data.message     AS MESSAGE
  FROM AQ$MESG_IN_QUEUE_TBL q;
/
GRANT EXECUTE ON MSG_AQ_ADMIN.ENQUEUE_MESG TO MSG_AQ_USER;
CREATE SYNONYM  MSG_AQ_USER.ENQUEUE_MESG FOR MSG_AQ_ADMIN.ENQUEUE_MESG;
